@@ -20,7 +20,7 @@ use crate::formatter::gas::mem_size_tbl::MEM_SIZE_TBL;
 use crate::formatter::gas::regs::*;
 use crate::formatter::instruction_internal::get_address_size_in_bytes;
 use crate::formatter::num_fmt::*;
-use crate::formatter::regs_tbl::REGS_TBL;
+use crate::formatter::regs_tbl_ls::REGS_TBL;
 use crate::formatter::*;
 use crate::iced_constants::IcedConstants;
 use crate::iced_error::IcedError;
